﻿namespace EPSON_SOCKET
{
    using System;
    using System.ComponentModel;
    using System.Drawing;
    using System.IO;
    using System.Net;
    using System.Net.Sockets;
    using System.Runtime.CompilerServices;
    using System.Text;
    using System.Text.RegularExpressions;
    using System.Threading;
    using System.Windows.Forms;

    public class EPSON_SOCKET : UserControl
    {
        private Thread thd_conn;
        private Thread thd_recv;
//        private bool isConnection = false;
        private bool timerStart = false;
        private IPAddress ip;
        private Socket clientSocket;
        private int port_value = 0x138a;
        private string ip_addr = "127.0.0.1";
        private bool init_success = false;
        private bool rec_start_flag = false;
        private string send_data = string.Empty;
        private int pickCounter = 0;
        private long totalPickCounter = 0L;
        private int place_counter = 0;
        private int transfer_counter = 0;
        private double x_val = 0.0;
        private double y_val = 0.0;
        private int groove_num;
        private int col_num;
        private double groove_to_groove;
        private double col_to_col;
        private double bar_length;
        private double col_step;
        private int bal_step;
        private double boder;
        private int in_bar_a;
        private int in_bar_b;
        private int rem_num;
        private int select_num;
        private int weightStep;
        private int weightCounter;
        private int weightFalseCounter;
        private int weightFalseMax;
        private int clearSpace;
        private int boatNum = 0;
//      private string warnMsg = string.Empty;      //收到警告消息.
        private string statusMsg = string.Empty;    //收到状态消息.
        private string node_time = "KFC";
        private int f_send_status = 0;
        private double resend_weight_value = 0.0;
        private IContainer components = null;
        private TextBox txtMsg;
        private bool is_admin;          //是否是管理端口.
        private bool is_reconnect = true;//是否开启自动重联.
        private bool bFirstConnect = true;//是否是第一次连接.

        public event BtnClickHandle rec_data_refresh;
        public event EventHandler OnStateChanged;
        private static object _lock = new object();

        public EPSON_SOCKET()
        {
            this.InitializeComponent();
            this.createLogDir();
        }

        private void createLogDir()
        {
            string logDir = System.Environment.CurrentDirectory + "\\Logs";
            Directory.CreateDirectory(logDir);
        }

        private void saveLog(string str)
        {
            string logDir = System.Environment.CurrentDirectory + "\\Logs\\";
            string logFile = logDir + DateTime.Today.ToString("yyyy-MM-dd") + ".log";
            string text = "【" + DateTime.Now.ToLongTimeString() + "】" + str + Environment.NewLine;
            try
            {
                FileStream fs = File.Open(logFile, FileMode.Append);
                StreamWriter sw = new StreamWriter(fs);
                sw.Write(text);
                sw.Close();
                fs.Close();
            }
            catch (Exception ex) {}
        }

        private void com_timer_Tick(object sender, EventArgs e)
        {
            this.connection_epson();
        }

        private void connection_epson()
        {
            this.init_epson_com();
            try
            {
                if (this.init_success)
                {
                    if (this.clientSocket == null)
                    {
                        this.clientSocket = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
                        this.clientSocket.Connect(new IPEndPoint(this.ip, this.port_value));
                        this.clientSocket.SendTimeout = 5000;
                        // 管理端口自动发送登录命令.
                        if ((this.is_admin) && (this.clientSocket.Connected))
                        {
                            this.Login();
                        }
                    }
                    else if (!this.clientSocket.Connected)
                    {
                        //如果不是自动重联.
                        if (!Is_Reconnect)
                        {
                            // 如果不是首次连接,那么直接返回不再连接.
                            if (!bFirstConnect)
                                return;
                        }

                        this.clientSocket.Dispose();
                        this.clientSocket = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
                        this.clientSocket.Connect(new IPEndPoint(this.ip, this.port_value));
                        this.clientSocket.SendTimeout = 5000;
                        // check if the last state has been changed item value content.
                        if (this.OnStateChanged != null) this.OnStateChanged(false, null);
                        // 管理端口自动发送登录命令.
                        if ((this.is_admin) && (this.clientSocket.Connected))
                        {
                            bFirstConnect = false;
                            this.Login();
                        }

                    }
                    if (((this.clientSocket != null) && this.clientSocket.Connected) && !this.rec_start_flag)
                    {
                        this.rec_start_flag = true;
                        this.thd_recv = new Thread(new ThreadStart(this.tcp_rec));
                        this.thd_recv.Start();
                        // check if the last state has been changed item value content.
                        if (this.OnStateChanged != null) this.OnStateChanged(STATES.STATUS_CONNECTED,null);
                    }
                }
            }
            catch (Exception exception)
            {
                if (!Is_Reconnect)
                {
                    // 连接不上,并且不开启自动重联,不再连接接.
                    bFirstConnect = false;
                }

                // if (exception.ErrorCode == 10060) bFirstConnect = false;
                this.print_msg("连接EPSON机器人异常:" + exception.Message);
            }
        }

        // 发送Login指令给设备.
        private void Login()
        {
            String cmd = String.Format("$login,1");
            this.tcp_send(cmd);
            this.print_msg("发送登录消息:" + cmd);
        }

        //发送GetStatus消息.
        public void sendGetStatus()
        {
            if (this.IsConnection)
            {
                // 00100000001
                String cmd = String.Format("$getstatus");
                this.tcp_send(cmd);
                this.print_msg("发送获取状态命令:" + cmd);
            }
        }

        public void Delay(int mm)
        {
            DateTime now = DateTime.Now;
            while (now.AddMilliseconds((double) mm) > DateTime.Now)
            {
                Application.DoEvents();
                System.Threading.Thread.Sleep(10);
            }
        }

        protected override void Dispose(bool disposing)
        {
            if (disposing && (this.components != null))
            {
                this.components.Dispose();
            }
            base.Dispose(disposing);
        }

        private void EPSON_SOCKET_Load(object sender, EventArgs e)
        {
            this.txtMsg.Size = base.Size;
        }

        private void EPSON_SOCKET_Resize(object sender, EventArgs e)
        {
            this.txtMsg.Size = base.Size;
        }

        private void init_epson_com()
        {
            this.init_success = true;
            try
            {
                this.ip = IPAddress.Parse(this.ip_addr);
            }
            catch
            {
                this.init_success = false;
                MessageBox.Show("IP地址格式错误！");
            }
            if (this.port_value > 0xffff)
            {
                this.init_success = false;
                MessageBox.Show("端口号设置操作范围！");
            }
        }

        private void InitializeComponent()
        {
            this.txtMsg = new System.Windows.Forms.TextBox();
            this.SuspendLayout();
            // 
            // txtMsg
            // 
            this.txtMsg.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
            this.txtMsg.Location = new System.Drawing.Point(0, 0);
            this.txtMsg.Multiline = true;
            this.txtMsg.Name = "txtMsg";
            this.txtMsg.ReadOnly = true;
            this.txtMsg.ScrollBars = System.Windows.Forms.ScrollBars.Vertical;
            this.txtMsg.Size = new System.Drawing.Size(100, 23);
            this.txtMsg.TabIndex = 0;
            // 
            // EPSON_SOCKET
            // 
            this.AutoScaleDimensions = new System.Drawing.SizeF(6F, 13F);
            this.AutoScaleMode = System.Windows.Forms.AutoScaleMode.Font;
            this.Controls.Add(this.txtMsg);
            this.Name = "EPSON_SOCKET";
            this.Size = new System.Drawing.Size(272, 119);
            this.Load += new System.EventHandler(this.EPSON_SOCKET_Load);
            this.Resize += new System.EventHandler(this.EPSON_SOCKET_Resize);
            this.ResumeLayout(false);
            this.PerformLayout();

        }

        private void print_msg(string str)
        {
            if (this.txtMsg.InvokeRequired)
            {
                SetTextCallback method = new SetTextCallback(this.print_msg);
                base.Invoke(method, new object[] { str });
            }
            else
            {
                string text = this.txtMsg.Text;
                this.txtMsg.Text = text + "【" + DateTime.Now.ToLongTimeString() + "】" + str + Environment.NewLine;
                this.txtMsg.Select(this.txtMsg.Text.Length - 1, 0);
                this.txtMsg.ScrollToCaret();
            }

            this.saveLog(str);
        }

        // 管理员连接解码.
        private void rec_data_decode_admin(String str)
        {
            String cmdType = str.Substring(0, 1);
            str = str.Substring(1);
            String[] result = str.Split(',');
            if ((result == null) || (result.Length <= 0)) return;
            if (cmdType == "#")
            {
                switch (result[0])
                {
                    // 登录返回.
                    case "login":
                        {
                            if (result[1] == "0")
                            {
                                this.sendGetStatus();
                                if (this.OnStateChanged != null)
                                    this.OnStateChanged(STATES.STATUS_LOGIN_OK, new EventArgsCmd(str));
                            }
                            else
                            {
                                // 打印登录失败返回消息.
                                this.print_msg("登录失败:" + str);
                            }
                        } break;
                    // 获取状态.
                    case "getstatus":
                        {
                            this.statusMsg = result[1] + "," + result[2];
                        } break;
                    // 启动回复
                    case "start":
                        {
                            if (result[1] == "0")
                            {
                                // 为防止连接时端口还没有开,这里先等待500毫秒再连接数据端口.
                                System.Threading.Thread.Sleep(500);
                                this.OnStateChanged(STATES.STATUS_START_SUCCEDSS, new EventArgsCmd(str));
                            }
                        } break;
                    // 停止回复
                    case "stop":
                        {
                            // if (result[1] == "0")
                            {
                                this.OnStateChanged(STATES.STATUS_STOP_SUCCESS, new EventArgsCmd(str));
                            }
                        }
                        break;
                }
            }
            else if (cmdType == "!")
            {
                this.print_msg("命令失败:" + str);
            }
        }

        // 普通通信解码.
        private void rec_data_decode_normal(String str)
        {
                        String cmdType = str.Substring(0, 1);
            str = str.Substring(1);
            String[] result = str.Split('=');
            if ((result == null) || (result.Length <= 0)) return;
            if (cmdType == "#")
            {
                switch (result[0])
                {
                    case "statusmsg":
                        // this.statusMsg = result[1];
                        this.print_msg(result[1]);
                        break;
                }
            }

        }

        // 解析接收到的数据.
        private void rec_data_decode(string str)
        {
            str = Regex.Replace(str, @"[\r\n]", "");
            if (this.is_admin)
            {
                // 解析控制指令.
                rec_data_decode_admin(str);
            }
            else
            {
                // 解析数据内容.
                rec_data_decode_normal(str);
            }


        }

        private void tcp_rec()
        {
            try
            {
                if (this.clientSocket == null)
                {
                    this.print_msg("clientSocket为空!");
                    return;
                }

                if (!this.clientSocket.Connected)
                {
                    this.print_msg("clientSocket未连接!");
                    return;
                }

                // loop to received data now.
                while (this.rec_start_flag)
                {
                    string str = "";
                    byte[] buffer = new byte[1024];
                    int count = this.clientSocket.Receive(buffer, buffer.Length, SocketFlags.None);
                    if (count > 0)
                    {
                        int pos = -1;
                        str = str + Encoding.Default.GetString(buffer, 0, count);
                        while ((pos = str.IndexOf("\r\n")) > 0)
                        {
                            String item = str.Substring(0, pos);
                            this.rec_data_decode(item);
                            this.print_msg("服务端发来消息：" + item);
                            if (this.rec_data_refresh != null)
                            {
                                this.rec_data_refresh(new Button(), new EventArgsCmd(item));
                            }

                            // skip current item now.
                            str = str.Substring(pos+2);
                        }

                    }
                }
            }
            catch (Exception exception)
            {
                this.print_msg("接收异常：" + exception.Message);
            }
            finally
            {
                this.rec_start_flag = false;
                this.print_msg("服务器断开连接......");
            }
        }

        public void tcp_send(string sentstr)
        {
            try
            {
                if ((this.clientSocket != null) && this.clientSocket.Connected)
                {
                    string s = string.Empty;
                    s = sentstr + Environment.NewLine;
                    lock(_lock)
                    {
                        // 锁定并发送数据,为了防止同时发送数据导致冲突.
                        this.clientSocket.Send(Encoding.UTF8.GetBytes(s));
                    }

                    this.print_msg("发送：" + s);
                }
            }
            catch
            {
            }
        }

        private void timer_tick()
        {
            while (true)
            {
                // this.print_msg("检查机器人通信状态");
                this.connection_epson();
                this.Delay(500);
            }
        }

        public bool IsConnection
        {
            get
            {
                return ((this.clientSocket != null) && this.clientSocket.Connected);
            }
        }

        public bool TimerStart
        {
            get
            {
                return this.timerStart;
            }
            set
            {
                this.timerStart = value;
                this.bFirstConnect = true;
                if (value)
                {
                    this.thd_conn = new Thread(new ThreadStart(this.timer_tick));
                    this.thd_conn.Start();
                }
                if ((this.clientSocket != null) && !value)
                {
                    try
                    {
                        // 先停止接收线程.
                        if (this.thd_recv != null)
                        {
                            this.thd_recv.Abort();
                            this.thd_recv = null;
                        }
                        this.thd_conn.Abort();
                        this.clientSocket.Dispose();
                    }
                    catch (Exception)
                    {
                    }
                }
            }
        }

        public int Port_value
        {
            get
            {
                return this.port_value;
            }
            set
            {
                this.port_value = value;
            }
        }

        public string Ip_addr
        {
            get
            {
                return this.ip_addr;
            }
            set
            {
                this.ip_addr = value;
            }
        }

        public string Send_data
        {
            get
            {
                return this.send_data;
            }
            set
            {
                this.send_data = value;
                this.tcp_send(this.send_data);
            }
        }

        public int PickCounter
        {
            get
            {
                return this.pickCounter;
            }
        }

        public long TotalPickCounter
        {
            get
            {
                return this.totalPickCounter;
            }
        }

        public int Place_counter
        {
            get
            {
                return this.place_counter;
            }
        }

        public int Transfer_counter
        {
            get
            {
                return this.transfer_counter;
            }
        }

        public double X_val
        {
            get
            {
                return this.x_val;
            }
        }

        public double Y_val
        {
            get
            {
                return this.y_val;
            }
        }

        public int Groove_num
        {
            get
            {
                return this.groove_num;
            }
            set
            {
                this.groove_num = value;
            }
        }

        public int Col_num
        {
            get
            {
                return this.col_num;
            }
            set
            {
                this.col_num = value;
            }
        }

        public double Groove_to_groove
        {
            get
            {
                return this.groove_to_groove;
            }
            set
            {
                this.groove_to_groove = value;
            }
        }

        public double Col_to_col
        {
            get
            {
                return this.col_to_col;
            }
            set
            {
                this.col_to_col = value;
            }
        }

        public double Bar_length
        {
            get
            {
                return this.bar_length;
            }
            set
            {
                this.bar_length = value;
            }
        }

        public double Col_step
        {
            get
            {
                return this.col_step;
            }
            set
            {
                this.col_step = value;
            }
        }

        public int Bal_step
        {
            get
            {
                return this.bal_step;
            }
            set
            {
                this.bal_step = value;
            }
        }

        public double Boder
        {
            get
            {
                return this.boder;
            }
            set
            {
                this.boder = value;
            }
        }

        public int In_bar_a
        {
            get
            {
                return this.in_bar_a;
            }
            set
            {
                this.in_bar_a = value;
            }
        }

        public int In_bar_b
        {
            get
            {
                return this.in_bar_b;
            }
            set
            {
                this.in_bar_b = value;
            }
        }

        public int Rem_num
        {
            get
            {
                return this.rem_num;
            }
            set
            {
                this.rem_num = value;
            }
        }

        public int Select_num
        {
            get
            {
                return this.select_num;
            }
            set
            {
                this.select_num = value;
            }
        }

        public int WeightStep
        {
            get
            {
                return this.weightStep;
            }
            set
            {
                this.weightStep = value;
            }
        }

        public int WeightCounter
        {
            get
            {
                return this.weightCounter;
            }
            set
            {
                this.weightCounter = value;
            }
        }

        public int WeightFalseCounter
        {
            get
            {
                return this.weightFalseCounter;
            }
            set
            {
                this.weightFalseCounter = value;
            }
        }

        public int WeightFalseMax
        {
            get
            {
                return this.weightFalseMax;
            }
            set
            {
                this.weightFalseMax = value;
            }
        }

        public int ClearSpace
        {
            get
            {
                return this.clearSpace;
            }
            set
            {
                this.clearSpace = value;
            }
        }

        public int BoatNum
        {
            get
            {
                return this.boatNum;
            }
            set
            {
                this.boatNum = value;
            }
        }

        public string Node_time
        {
            get
            {
                return this.node_time;
            }
            set
            {
                this.node_time = value;
            }
        }

        public int F_send_status
        {
            get
            {
                return this.f_send_status;
            }
            set
            {
                this.f_send_status = value;
            }
        }

        public double Resend_weight_value
        {
            get
            {
                return this.resend_weight_value;
            }
            set
            {
                this.resend_weight_value = value;
            }
        }

        public Boolean Is_admin
        {
            get
            {
                return this.is_admin;
            }
            set
            {
                is_admin = value;
            }
        }

        public Boolean Is_Reconnect
        {
            get
            {
                return this.is_reconnect;
            }
            set
            {
                is_reconnect = value;
            }
        }

        public string StatusMsg
        {
            get
            {
                return this.statusMsg;
            }
            set
            {
                this.statusMsg = value;
            }
        }

        public delegate void BtnClickHandle(object sender, EventArgs e);

        private delegate void SetTextCallback(string text);
    }
}

