﻿using EPSON_SOCKET;
using Maticsoft.DBUtility;
using PCHMI;
using PLCCommunication;
using PLCCommunication.Common;

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using ZWGXPICK_SYS.DAL;
using ZWGXPICK_SYS.SerialPort;
using ZWGXZWGXPICK_SYS.DAL;

namespace ZWGXPICK_SYS
{
    public partial class MainForm : Form
    {
        // 行列最大数量.
        private static int maxItem = 301;

        private Region myRegion;
        private SolidBrush myBrush = new SolidBrush(Color.LightSeaGreen);
        private GraphicsPath[,] mlbMatrix = new GraphicsPath[301, 301];
        private defPart[,] mPart = new defPart[301, 301];   //每个产品显示情况定义.
        private defCorner[] mCorner = new defCorner[4];     //四个顶角支撑位定义.

        // 外框和中心孔.
        private GraphicsPath mlbDashBorder = new GraphicsPath(); //外框.
        private GraphicsPath mlbCenterHole = new GraphicsPath(); //中心孔.

        private int boatDashType = 1;                     //舟皿类型,0:平板舟皿,1:槽形舟皿,2:圆形舟皿.
        private int productType = 1;                     //0:矩形,1:三角形.
        private static float half_sqrt3 = 1.7320508075688772f * 0.5f; //等边三角形外切圆半径公式√3/2.
        private PointF drawStartpos = new PointF(10, 10); //仿真绘制左上角起始点位置.

        private int nPartsPerTray = 0;                     //每盘最终摆盘数量.
        private short nRowNo = 0;                     //每行数量.
        private short nColNo = 0;                     //每列数量.

        // 模拟生产相关参数.
        private bool bisRunning = false;                 //当前是否正在模拟生产.
        private int nMaxPlaceTimes = 0;                     //通过当前摆放规则,计算出一盘最大摆放次数.
        private int nCurPlacedTimes = 0;                     //记录当前已经摆放次数.
                                                             //      private int nPlanPlaceNum  = 0;                     //下次摆放次下标.

        private int nRowPlaceNo = 0;                     //每行最多可以摆放多少次.
        private int nColPlaceNo = 0;                     //每列最多可以摆放多少次.
                                                         //         private int nProdColIndex  = 0;                  //当前摆放X.
                                                         //         private int nProdRowIndex  = 0;                  //当前摆放Y.
        private int nPlaceType = 0;                     //摆放方式(0:对齐摆放,1:插空摆放)
        private int nMatrixType = 0;                     //阵列类型.
        private int nXDirectionNum = 0;                     //X方向一次取件数量.
        private int nYDirectionNum = 0;                     //Y方向一次取件数量.

        // 机器人上发参数.
        private float startAndroidX = 0;                     //机器人上发,android起始X坐标.
        private float startAndroidY = 0;                     //机器人上发,android起始Y坐标.

        private List<Point> lstPlaceXY = new List<Point>(); //记录机器人摆放位置,模拟仿真的时候会用到.

        private FormWindowState lastState = FormWindowState.Normal;

        private float X;
        private float Y;
        private ZWGXPICK_SYS.Model.PRODUCT_ITEM_CONFIG product = null; //当前选中的产品信息.

        /// <summary>
        /// 重量设类
        /// </summary>
        private WeightEquipment weightEquipment = null;
        /// <summary>
        /// 设置单重上限
        /// </summary>
        private decimal setSingleUpperLimit = 0;
        /// <summary>
        /// 设置调整上限
        /// </summary>
        private decimal setAdjustmentUpperLimit = 0;
        /// <summary>
        /// 设置一个标准重量
        /// </summary>
        private decimal setStandardWeight = 0;
        /// <summary>
        /// 设置调整下限
        /// </summary>
        private decimal setAdjustmentLowerLimit = 0;
        /// <summary>
        /// 设置单重下限
        /// </summary>
        private decimal setSingleLowerLimit = 0;

        public MainForm()
        {
            InitializeComponent();
            // 生成行列所有元素内容.
            for (int row = 0; row < 301; row++)
            {
                for (int col = 0; col < 301; col++)
                {
                    // 初始化所有显示单元.
                    this.mPart[row, col] = new defPart();
                    this.mlbMatrix[row, col] = new GraphicsPath();
                }
            }
            //RICADO.MettlerToledo.MettlerToledoDevice mettlerToledoDevice = new RICADO.MettlerToledo.MettlerToledoDevice(RICADO.MettlerToledo.ConnectionMethod.Ethernet,RICADO.MettlerToledo.ProtocolType.SICS,"111",10);  
        }

        private void setTag(Control cons)
        {
            foreach (Control con in cons.Controls)
            {
                con.Tag = con.Width + ":" + con.Height + ":" + con.Left + ":" + con.Top + ":" + con.Font.Size;
                if (con.Controls.Count > 0)
                    setTag(con);
            }
        }

        private void setControls(float newx, float newy, Control cons)
        {
            foreach (Control con in cons.Controls)
            {
                if (con.Tag == null) continue;
                string[] mytag = con.Tag.ToString().Split(new char[] { ':' });
                float a = Convert.ToSingle(mytag[0]) * newx;
                con.Width = (int)a;
                a = Convert.ToSingle(mytag[1]) * newy;
                con.Height = (int)(a);
                a = Convert.ToSingle(mytag[2]) * newx;
                con.Left = (int)(a);
                a = Convert.ToSingle(mytag[3]) * newy;
                con.Top = (int)(a);
                Single currentSize = Convert.ToSingle(mytag[4]) * Math.Min(newx, newy);
                con.Font = new Font(con.Font.Name, currentSize, con.Font.Style, con.Font.Unit);
                if (con.Controls.Count > 0)
                {
                    setControls(newx, newy, con);
                }
            }

        }

        void Form1_Resize(object sender, EventArgs e)
        {
            float newx = (this.Width) / X;
            float newy = this.Height / Y;
            if ((this.WindowState != FormWindowState.Minimized) && (lastState != FormWindowState.Minimized))
            {
                setControls(newx, newy, this);
            }

            // this.Text = this.Width.ToString() + " " + this.Height.ToString();
            lastState = this.WindowState;
        }

        [DllImport("USER32.DLL")]
        public static extern int GetSystemMenu(int hwnd, int bRevert);
        [DllImport("USER32.DLL")]
        public static extern int RemoveMenu(int hMenu, int nPosition, int wFlags);
        /// <summary>
        /// 返回值，非零表示成功，零表示失败。
        /// </summary>
        /// <param name="iHWND">窗口的句柄</param>
        /// <returns>是否成功</returns>
        public int RemoveXButton(int iHWND)
        {
            int iSysMenu;
            const int MF_BYCOMMAND = 0x400; //0x400-关闭
            iSysMenu = GetSystemMenu(this.Handle.ToInt32(), 0);
            return RemoveMenu(iSysMenu, 6, MF_BYCOMMAND);
        }

        // 窗口加载.
        // weight_mode,clear_burr,flush_wild,get_mode,get_tool,weight_interval,cleanmode_interval,continuity_unqualified;1;2;3;4;5;6;7;8;
        private void frmMain_Load(object sender, EventArgs e)
        {
            RemoveXButton(this.Handle.ToInt32());
            PLC_Communication.START((Control)this, System.Reflection.Assembly.GetExecutingAssembly(), null);
            this.Resize += new EventHandler(Form1_Resize);
            X = this.Width;
            Y = this.Height;
            setTag(this);
            Form1_Resize(new object(), new EventArgs());//x,y可在实例化时赋值,最后这句是新加的，在MDI时有用
            this.CenterToScreen();

            // 加载ini配置.
            LoadIniConfig();
            if (this.product != null)
            {
                LoadProductById(this.product.id);
            }

            // 更新警告日志显示.
            updateWarnLogView();
            btnOpenAndroid.Enabled = true;
            btnStartProduct.Enabled = false;
            btnCloseAndroid.Enabled = false;
            btnStopProduct.Enabled = false;
            btnStopNow.Enabled = false;
            btnErrorReset.Enabled = false;
            btnPoint.Enabled = false;
            btnAndroidIO.Enabled = false;

            // 初始化所有的Trigger.
            initTriggerList();
            this.createLogDir();

         
        }

        // 绘制函数.
        private void picMatirxMain_Paint(object sender, PaintEventArgs e)
        {
            e.Graphics.SmoothingMode = SmoothingMode.HighQuality;
            // try to draw the special dash box item border now.
            e.Graphics.DrawPath(Pens.Blue, this.mlbDashBorder);
            // try to draw the special center hold item now.
            e.Graphics.DrawPath(Pens.Red, this.mlbCenterHole);

            for (int row = 0; row < maxItem; row++)
            {
                for (int col = 0; col < maxItem; col++)
                {
                    if (this.mPart[row, col].bVisible)
                    {
                        this.myRegion = new Region(this.mlbMatrix[row, col]);
                        switch (this.mPart[row, col].nColor)
                        {
                            case 0:
                                this.myBrush = new SolidBrush(Color.Blue); //默认.
                                break;

                            case 1:
                                this.myBrush = new SolidBrush(Color.Red);  //已摆.
                                break;

                            case 2:
                                this.myBrush = new SolidBrush(Color.Gray); //其他.
                                break;
                        }

                        e.Graphics.FillRegion(this.myBrush, this.myRegion);
                        // e.Graphics.DrawPath(Pens.Black, this.mlbMatrix[row, col]);
                    }
                }
            }
        }

        // 加载ini配置信息.
        private void LoadIniConfig()
        {
            IniFile ini = new IniFile(PubConstant.config_file);
            int id = ini.ReadInteger("global", "product_id", 0);
            if (id != 0)
            {
                product = new PRODUCT_ITEM_CONFIG().GetModel(id);
                this.txt_product_name.Text = (product == null) ? "" : product.product_name;
            }

            // try to get the other content item value try to get the other content item value.
            this.cb_weight_mode.SelectedIndex = ini.ReadInteger("global", "cb_weight_mode", 0);
            this.cb_clear_burr.SelectedIndex = ini.ReadInteger("global", "cb_clear_burr", 0);
            this.cb_flush_wild.SelectedIndex = ini.ReadInteger("global", "cb_flush_wild", 0);
            this.cb_get_mode.SelectedIndex = ini.ReadInteger("global", "cb_get_mode", 0);
            this.cb_get_tool.SelectedIndex = ini.ReadInteger("global", "cb_get_tool", 0);
            this.txt_weight_interval.Text = ini.ReadString("global", "txt_weight_interval", "");
            this.txt_cleanmode_interval.Text = ini.ReadString("global", "txt_cleanmode_interval", "");
            this.txt_continuity_unqualified.Text = ini.ReadString("global", "txt_continuity_unqualified", "");

            //this.txt_standard_weight.Text = ini.ReadString("global", "txt_standard_weight", "");
            //this.txt_qualified_allow_error.Text = ini.ReadString("global", "txt_qualified_allow_error", "");
            //this.txt_single_weight_allow_error.Text = ini.ReadString("global", "txt_single_weight_allow_error", "");
            this.txt_plan_placed_num.Text = ini.ReadString("global", "txt_plan_placed_num", "1");
            if (this.txt_plan_placed_num.Text == "") this.txt_plan_placed_num.Text = "1";

            int nSpeed = ini.ReadInteger("global", "txt_run_speed", 5);
            if ((nSpeed < 5) || (nSpeed > 100)) nSpeed = 5;
            this.txt_run_speed.Text = nSpeed.ToString();

            //重量计算参数
            setSingleUpperLimit = decimal.Parse(ini.ReadString("weight_RS232", "setSingleUpperLimit", "0"));
            setAdjustmentUpperLimit = decimal.Parse(ini.ReadString("weight_RS232", "setAdjustmentUpperLimit", "0"));
            setStandardWeight = decimal.Parse(ini.ReadString("weight_RS232", "setStandardWeight", "0"));
            setAdjustmentLowerLimit = decimal.Parse(ini.ReadString("weight_RS232", "setAdjustmentLowerLimit", "0"));
            setSingleLowerLimit = decimal.Parse(ini.ReadString("weight_RS232", "setSingleLowerLimit", "0"));

            //重量设备 初始化
            var baud = ini.ReadString("weight_RS232", "baud");
            var comName = ini.ReadString("weight_RS232", "comName");
            weightEquipment = new WeightEquipment(comName,int.Parse(baud));
            weightEquipment.MessageSend += WeightEquipment_MessageSend;
            weightEquipment.WeightSend += WeightEquipment_WeightSend;
        }

        // 写ini配置信息.
        private void WriteIniString(string section, string name, string value)
        {
            IniFile ini = new IniFile(PubConstant.config_file);
            ini.WriteString(section, name, value);
        }


        // 产品参数.
        private void btnProduct_Click(object sender, EventArgs e)
        {
            Product_Data_Form form = new Product_Data_Form();
            form.ShowDialog(this);
        }

        // 点位配置.
        private void btnPoint_Click(object sender, EventArgs e)
        {
            /*
            // 如果当前机器人是打开的,首先关闭当前机器人连接.
            if (this.epsonSocket1.IsConnection)
            {
                // 停止机器人运行.
                btnStopProduct_Click(sender,e);
                // 关闭机器人连接.
                btnCloseAndroid_Click(sender,e);
            } */

            // 显示示教窗口.
            Point_Data_Form form = new Point_Data_Form(this.epsonSocket1);
            form.ShowDialog(this);
        }

        // 换舟参数.
        private void btnChangeDashParam_Click(object sender, EventArgs e)
        {
            Dash_Change_Form form = new Dash_Change_Form();
            form.ShowDialog(this);
        }

        // PLC监控.
        private void btnPLC_Click(object sender, EventArgs e)
        {
            PLC_IO_Form form = new PLC_IO_Form();
            form.ShowDialog(this);
        }

        // 称重查询.
        private void btnWeight_Click(object sender, EventArgs e)
        {
            Weight_Data_Form form = new Weight_Data_Form();
            form.ShowDialog(this);
        }

        // 报警查询.
        private void btnWarning_Click(object sender, EventArgs e)
        {
            Alarm_Log_Form form = new Alarm_Log_Form();
            form.ShowDialog(this);
        }

        // 退出程序
        private void btnExit_Click(object sender, EventArgs e)
        {
            {
                MessageBox.Show("退出程序前机器需停止运转!", "操作确认", MessageBoxButtons.OK);
            }

            {
                DialogResult dr = MessageBox.Show("确定要退出控制软件吗？", "操作确认", MessageBoxButtons.YesNo);
                if (dr == DialogResult.Yes)
                {
                    this.Close();
                }
            }
        }

        // 配置机器人IP地址等参数
        private void btnIpConfig_Click(object sender, EventArgs e)
        {
            frmAndroidIP form = new frmAndroidIP();
            form.ShowDialog(this);
        }

        private void btnOpenAndroid_Click(object sender, EventArgs e)
        {
            if (MessageBox.Show("您好，确定要打开机器人吗？", "操作确认", MessageBoxButtons.OKCancel) == DialogResult.OK)
            {
                btnStartProduct.Enabled = true;
                btnCloseAndroid.Enabled = true;
                btnStopProduct.Enabled = true;
                btnStopNow.Enabled = true;
                btnErrorReset.Enabled = true;
                btnPoint.Enabled = true;
                btnAndroidIO.Enabled = true;
                IniFile ini = new IniFile(PubConstant.config_file);
                String ip = ini.ReadString("android", "ip", "127.0.0.1");
                int nAdminPort = ini.ReadInteger("android", "admin_port", 0);
                if (String.IsNullOrEmpty(ip) || nAdminPort == 0)
                {
                    MessageBox.Show("机器人参数不正常,请先配置正确的机器人IP地址和控制端口!");
                    return;
                }

                // 启动机器人连接.
                this.epsonSocket1.Ip_addr = ip;
                this.epsonSocket1.Port_value = nAdminPort;
                this.epsonSocket1.TimerStart = true;
            }

        }

        // 连接机器人数据端口.
        private void ConnectDataSocket()
        {
            IniFile ini = new IniFile(PubConstant.config_file);
            String ip = ini.ReadString("android", "ip", "127.0.0.1");
            int nDataPort = ini.ReadInteger("android", "data_port", 0);
            if (String.IsNullOrEmpty(ip) || nDataPort == 0)
            {
                MessageBox.Show("机器人参数不正常,请先配置正确的机器人IP地址和数据端口!");
                return;
            }

            // 启动机器人连接.
            this.epsonSocket2.Ip_addr = ip;
            this.epsonSocket2.Port_value = nDataPort;
            this.epsonSocket2.TimerStart = true;
        }


        // 连接状态改变时调用此函数.
        private void epsonSocket1_OnStateChanged(object sender, EventArgs e)
        {
            STATES status = (STATES)Convert.ToInt32(sender);
            switch (status)
            {
                case STATES.STATUS_CONNECTED: //连接成功
                    {

                        if (this.epsonSocket1.IsConnection)
                        {
                            //                             btnOpenAndroid.Enabled = false;
                            //                             btnStartProduct.Enabled = true;
                            //                             btnChangeContainer.Enabled = true;
                            //                             btnCloseAndroid.Enabled = true;
                            //                             btnStopProduct.Enabled = true;
                            //                             btnStopNow.Enabled = true;
                            //                             btnErrorReset.Enabled = true;

                            // 点位示例功能按钮.
                            btnPoint.Enabled = true;
                        }
                        else
                        {
                            //                             btnOpenAndroid.Enabled = true;
                            //                             btnStartProduct.Enabled = false;
                            //                             btnChangeContainer.Enabled = false;
                            //                             btnCloseAndroid.Enabled = false;
                            //                             btnStopProduct.Enabled = false;
                            //                             btnStopNow.Enabled = false;
                            //                             btnErrorReset.Enabled = false;
                        }
                    }
                    break;
                case STATES.STATUS_LOGIN_OK:
                    {
                        // 启动控制端口心跳计数器.
                        Timer_Getstatus.Enabled = true;
                        Timer_Getstatus.Start();
                        //                         this.btnStartProduct.Enabled = true;


                    }
                    break;
                case STATES.STATUS_START_SUCCEDSS:
                    {
                        // 机器人启动成功.
                        //                         this.btnStartProduct.Enabled = false;
                        //                         this.btnStopProduct.Enabled = true;

                        // 连接第二个数据端口.
                        this.ConnectDataSocket();
                    }
                    break;
                case STATES.STATUS_STOP_SUCCESS:
                    {
                        // 机器人停止成功.
                        //                         this.btnStartProduct.Enabled = true;
                        //                         this.btnStopProduct.Enabled = false;
                    }
                    break;
            }
        }

        // 数据通信端口事件响应.
        private void epsonSocket2_OnStateChanged(object sender, EventArgs e)
        {
            STATES status = (STATES)Convert.ToInt32(sender);
            switch (status)
            {
                case STATES.STATUS_CONNECTED: //连接成功
                    {
                        // 数据端口连接成功之后发送connect指令.
                        this.epsonSocket2.tcp_send("$connect");
                    }
                    break;
            }
        }

        // 关闭机器人.
        private void btnCloseAndroid_Click(object sender, EventArgs e)
        {

            if (MessageBox.Show("您好，确定要关闭机器人吗？", "操作确认", MessageBoxButtons.OKCancel) == DialogResult.OK)
            {
                // 关闭数据端口.
                if (this.epsonSocket2.IsConnection)
                    this.epsonSocket2.TimerStart = false;

                // 关闭控制端口.
                if (this.epsonSocket1.IsConnection)
                {
                    this.epsonSocket1.tcp_send("$Logout");
                    System.Threading.Thread.Sleep(100);
                    this.epsonSocket1.TimerStart = false;
                }

                btnOpenAndroid.Enabled = true;
                btnStartProduct.Enabled = false;
                btnCloseAndroid.Enabled = false;
                btnStopProduct.Enabled = false;
                btnStopNow.Enabled = false;
                btnErrorReset.Enabled = false;
                btnPoint.Enabled = false;
                btnAndroidIO.Enabled = false;

                // 清空机器人状态文本.
                this.lblAndroidErrorCode.Text = "";
                this.lblAndroidStatus.Text = "";
                this.lbl_android_startpos.Text = "";
            }

        }

        // 选择产品参数.
        private void btnSelectProduct_Click(object sender, EventArgs e)
        {
            Product_Data_Form form = new Product_Data_Form();
            form.ShowDialog(this);
            DialogResult dr = form.DialogResult;
            if (dr != DialogResult.OK)
            {
                product = null;
                return;
            }

            int id = form.id;
            LoadProductById(id);

            // 保存产品参数.
            WriteIniString("global", "product_id", product.id.ToString());
            WriteIniString("global", "product_name", product.product_name);

            // 如果选择了产品,上次摆放位置必须是1.
            this.txt_plan_placed_num.Text = "1";
        }

        // 加载产品数据
        private void LoadProductById(int id)
        {
            product = new PRODUCT_ITEM_CONFIG().GetModel(id);
            if (product == null) return;

            // update product name.
            String productName = "";
            if (this.product != null)
            {
                productName = this.product.product_name;
            }

            // 显示仿真绘制矩阵.
            this.ShowMatrix(product);
            btnEmulation.Enabled = true;

            // 初始化参数.
            this.nPlaceType = this.product.place_type;
            this.nMatrixType = this.product.matrix_type;
            this.nXDirectionNum = this.product.x_direction_num;
            this.nYDirectionNum = this.product.y_direction_num;

            // 对齐摆放.
            this.nColPlaceNo = ((this.nColNo) / this.nXDirectionNum);
            this.nRowPlaceNo = ((this.nRowNo) / this.nYDirectionNum);
            if ((int)ePlacementType.Chakong == this.nPlaceType)
            {
                // 先X后Y.
                if (this.nMatrixType == (int)eMatirxType.X2Y)
                {
                    // 插空摆放,行支持插空摆放,一个单位是摆放两组,所以数量这里要*2.
                    if (this.nXDirectionNum > 1)
                    {
                        this.nColPlaceNo = ((this.nColNo / (this.nXDirectionNum * 2)) * 2);
                        int lineSize = ((product.x_direction_num * 2) - 1);
                        if ((this.nColNo - (this.nColPlaceNo * product.x_direction_num)) >= lineSize)
                        {
                            // 最后还能多摆一行.
                            this.nColPlaceNo++;
                        }
                    }
                }
                else
                {
                    // 插空摆放,行支持插空摆放,一个单位是摆放两组,所以数量这里要*2.
                    if (this.nYDirectionNum > 1)
                    {
                        this.nRowPlaceNo = ((this.nRowNo / (this.nYDirectionNum * 2)) * 2);
                        int lineSize = ((product.y_direction_num * 2) - 1);
                        if ((this.nRowNo - (this.nRowPlaceNo * product.y_direction_num)) >= lineSize)
                        {
                            // 最后还能多摆一行.
                            this.nRowPlaceNo++;
                        }
                    }

                }
            }

            // 计算最终的摆放次数.
            this.nMaxPlaceTimes = this.nColPlaceNo * this.nRowPlaceNo;
            //          this.nCurPlacedTimes = 0;
            //          this.txt_last_placed_times.Text = this.nCurPlacedTimes.ToString();

            // 这四个参数是根据产品配置参数带过来的.
            this.txt_product_name.Text = productName;
            this.txt_standard_weight.Text = this.product.standard_weight.ToString();
            this.txt_qualified_allow_error.Text = this.product.qualified_allow_error.ToString();
            this.txt_single_weight_allow_error.Text = this.product.single_weight_allow_error.ToString();

            // 如果是圆形舟皿,直接使用中心点参数做为起始坐标.
            if (this.product.boat_dish_type == (int)eBoatDashType.DashCircle)
            {
                // 圆舟需要单独处理,首先需要将坐标修正到0,0.
                this.startAndroidX = (float)(this.product.boat_dash_center_x + (this.product.boat_dish_length * 0.5));
                this.startAndroidY = (float)(this.product.boat_dish_center_y - (this.product.boat_dish_length * 0.5));
            }
        }

        // 称重模式.
        private void cb_weight_mode_SelectedIndexChanged(object sender, EventArgs e)
        {
            int index = this.cb_weight_mode.SelectedIndex;
            WriteIniString("global", "cb_weight_mode", index.ToString());
        }

        // 清毛刺.
        private void cb_clear_burr_SelectedIndexChanged(object sender, EventArgs e)
        {
            int index = this.cb_clear_burr.SelectedIndex;
            WriteIniString("global", "cb_clear_burr", index.ToString());
        }

        // 是否吹气.
        private void cb_flush_wild_SelectedIndexChanged(object sender, EventArgs e)
        {
            int index = this.cb_flush_wild.SelectedIndex;
            WriteIniString("global", "cb_flush_wild", index.ToString());
        }

        // 抓取方式.
        private void cb_get_mode_SelectedIndexChanged(object sender, EventArgs e)
        {
            int index = this.cb_get_mode.SelectedIndex;
            WriteIniString("global", "cb_get_mode", index.ToString());
        }

        // 抓取工具.
        private void cb_get_tool_SelectedIndexChanged(object sender, EventArgs e)
        {
            int index = this.cb_get_tool.SelectedIndex;
            WriteIniString("global", "cb_get_tool", index.ToString());
        }

        // 称重频率.
        private void txt_weight_interval_TextChanged(object sender, EventArgs e)
        {
            String index = this.txt_weight_interval.Text;
            WriteIniString("global", "txt_weight_interval", index.ToString());
        }

        // 清模频率.
        private void txt_cleanmode_interval_TextChanged(object sender, EventArgs e)
        {
            String index = this.txt_cleanmode_interval.Text;
            WriteIniString("global", "txt_cleanmode_interval", index.ToString());
        }

        // 称重连续不合格.
        private void txt_continuity_unqualified_TextChanged(object sender, EventArgs e)
        {
            String index = this.txt_continuity_unqualified.Text;
            WriteIniString("global", "txt_continuity_unqualified", index.ToString());
        }

        // 标准重量:
        private void txt_standard_weight_TextChanged(object sender, EventArgs e)
        {
            String index = this.txt_standard_weight.Text;
            WriteIniString("global", "txt_standard_weight", index.ToString());
        }

        // 合格允许误差:
        private void txt_qualified_allow_error_TextChanged(object sender, EventArgs e)
        {
            String index = this.txt_qualified_allow_error.Text;
            WriteIniString("global", "txt_qualified_allow_error", index.ToString());
        }

        // 单重调整允许误差:
        private void txt_single_weight_allow_error_TextChanged(object sender, EventArgs e)
        {
            String index = this.txt_single_weight_allow_error.Text;
            WriteIniString("global", "txt_single_weight_allow_error", index.ToString());
        }

        // 接收到下位机的数据了.
        private void epsonSocket1_rec_data_refresh(object sender, EventArgs e)
        {
            updateAndroidStatus(this.epsonSocket1.StatusMsg);
        }

        // 上次摆放次数.
        private void txt_last_placed_times_TextChanged(object sender, EventArgs e)
        {
        }

        // 改变速度.
        private void txt_run_speed_Leave(object sender, EventArgs e)
        {
            int nSpeed = Utils.TryToInt32(this.txt_run_speed.Text);
            if (nSpeed < 5) nSpeed = 5;
            if (nSpeed > 100) nSpeed = 100;
            this.txt_run_speed.Text = nSpeed.ToString();
        }

        // 保存所有配置信息.
        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            WriteIniString("global", "txt_plan_placed_num", this.txt_plan_placed_num.Text);
            WriteIniString("global", "txt_run_speed", this.txt_run_speed.Text);

            PLCCommunicationFactory.Instance.ColseConnection();
        }

        // 数据端口有新的数据进来.
        private void epsonSocket2_rec_data_refresh(object sender, EventArgs e)
        {
            if (e == null) return;
            EventArgsCmd args = (EventArgsCmd)e;
            switch (args.Cmd)
            {
                case "config"://请求配置.
                    {
                        int nSpeed = Utils.TryToInt32(this.txt_run_speed.Text);
                        int planPlaceNum = Convert.ToInt32(this.txt_plan_placed_num.Text);
                        //发送配置参数给机器人.//称重模式,去毛刺,吹气,取件方式,取件工具,称重频率,清模频率,称重连续不合格,标准重量,合格允许误差,单重调整允许误差,上次摆放次数.
                        String cmd = String.Format("$config;czms={0};qmc={1};cq={2};qjfs={3};qjgj={4};czpl={5};qmpl={6};czlxbhg={7};bzzl={8:F};hgyxwc={9:F};dctzyxwc={10:F};scbfcs={11};sd={12};",
                                        this.cb_weight_mode.SelectedIndex, this.cb_clear_burr.SelectedIndex, this.cb_flush_wild.SelectedIndex, this.cb_get_mode.SelectedIndex,
                                        this.cb_get_tool.SelectedIndex, this.txt_weight_interval.Text, this.txt_cleanmode_interval.Text, this.txt_continuity_unqualified.Text,
                                        Convert.ToDouble(this.txt_standard_weight.Text), Convert.ToDouble(this.txt_qualified_allow_error.Text),
                                        Convert.ToDouble(this.txt_single_weight_allow_error.Text), planPlaceNum, nSpeed);
                        this.epsonSocket2.tcp_send(cmd);
                    }
                    break;
                case "placepoint"://机器人发首坐标.
                    {
                        // 收到机器人首坐标通知.
                        if (args.Result.Length >= 3)
                        {
                            Vector3d point = EventArgsCmd.ParseVector3d(args.Str);
                            if (this.boatDashType != (int)eBoatDashType.DashCircle)
                            {
                                // 上发机器人首坐标地址.
                                this.startAndroidX = (float)point.X;
                                this.startAndroidY = (float)point.Y;
                                String text = String.Format("机器人首坐标地址:({0},{1})", this.startAndroidX, this.startAndroidY);
                                this.lbl_android_startpos.Text = text;
                            }
                        }
                    }
                    break;
                case "placedatax"://请求坐标阵列.
                case "placedatay":
                case "placedatar":
                    {
                        /*
                        // 确认是否收到下位机发送的坐标数据.
                        if ((this.startAndroidX == (-int.MaxValue)) || (this.startAndroidY == (-int.MaxValue)))
                        {
                            if (DialogResult.Yes != MessageBox.Show("当前没有收到机器人发送来的起始坐标值,是否确认要开始生产!?", "警告", MessageBoxButtons.YesNo))
                            {
                                return;
                            }
                        }
                        */

                        String sendCmd = "pdr";
                        if (args.Cmd == "placedatax")
                        {
                            sendCmd = "pdx";
                        }
                        else if (args.Cmd == "placedatay")
                        {
                            sendCmd = "pdy";
                        }

                        // 根据摆放类型,获取生产坐标.
                        List<float> lstData = new List<float>();
                        GetMatrixPosition(args.Cmd, ref lstData);
                        if ((lstData.Count > 0) && (this.epsonSocket2.IsConnection))
                        {
                            int nMaxItem = 30;
                            int nGroup = lstData.Count / nMaxItem;
                            int nDiv = lstData.Count % nMaxItem;
                            int nSendGroup = (nDiv > 0) ? nGroup + 1 : nGroup;

                            for (int i = 0; i < nGroup; i++)
                            {
                                String str = "";
                                int startIndex = (i * nMaxItem);
                                for (int j = startIndex; j < (startIndex + nMaxItem); j++)
                                {
                                    str += String.Format("{0:F1};", lstData[j]);
                                }

                                // send current group data and sleep on moment item.
                                String cmd = String.Format("${0};{1};{2};{3};{4}", sendCmd, lstData.Count, nSendGroup, i + 1, str);
                                this.epsonSocket2.tcp_send(cmd);
                                System.Threading.Thread.Sleep(100);
                            }

                            // check if any div data item.
                            if (nDiv > 0)
                            {
                                String str = "";
                                int startIndex = nGroup * nMaxItem;
                                for (int j = startIndex; j < (startIndex + nDiv); j++)
                                {
                                    str += String.Format("{0:F1};", lstData[j]);
                                }
                                // send current group data and sleep on moment item.
                                String cmd = String.Format("${0};{1};{2};{3};{4}", sendCmd, lstData.Count, nSendGroup, nSendGroup, str);
                                this.epsonSocket2.tcp_send(cmd);
                            }

                        }
                    }
                    break;
                case "warnmsg"://发送警告信息.
                    {
                        // 如果有日志,即保存日志数据.
                        String WarnMsg = args.Result[1].ToString();
                        if (!String.IsNullOrEmpty(WarnMsg))
                        {
                            String log_content = WarnMsg;
                            ZWGXPICK_SYS.Model.ALARM_LOG model = new ZWGXPICK_SYS.Model.ALARM_LOG();
                            model.log_date = DateTime.Now.ToString("yyyy-MM-dd");
                            model.log_time = DateTime.Now.ToString("HH:mm:ss");
                            model.log_classify = "机器人报警";
                            model.log_content = log_content;
                            new ALARM_LOG().Add(model);
                            updateWarnLogView();
                        }
                    }
                    break;
                case "weightdata"://发送称重数据.
                    {
                        // 称重数据.
                        ZWGXPICK_SYS.Model.WEIGHT_LOG weight = new Model.WEIGHT_LOG();
                        weight.product_model = product.product_name;
                        weight.product_date = DateTime.Now.ToString("yyyy-MM-dd");
                        weight.product_time = DateTime.Now.ToString("HH:mm:ss");
                        weight.weight = args.Result[1].ToString();
                        weight.is_qualified = (args.Result[2] == "合格") ? 1 : 0;
                        new WEIGHT_LOG().Add(weight);
                    }
                    break;
                case "currentplacenum"://发送当前生产到哪个摆放位.
                    {
                        // 根据接收来的参数刷新整个显示阵列.
                        int placeTimes = Convert.ToInt32(args.Result[1]);
                        this.nCurPlacedTimes = (placeTimes - 1);
                        this.updateEnumationGraphicsMatrix();

                        // 判断当前盘是否已经摆放完毕,上位机是从0开始,比下位机要小1,所有这里加1再判断是否摆完.
                        if ((this.nCurPlacedTimes + 1) >= this.nMaxPlaceTimes)
                        {
                            // 重置当前位置计数器.
                            this.nCurPlacedTimes = 0;
                            // 重置所有产品颜色为0
                            for (int row = 0; row < maxItem; row++)
                            {
                                for (int col = 0; col < maxItem; col++)
                                {
                                    this.mPart[row, col].nColor = 0;
                                }
                            }

                            // 刷新空间显示.
                            this.picMatirxMain.Refresh();
                        }

                        // 保存ini配置信息.
                        IniFile ini = new IniFile(PubConstant.config_file);
                        ini.WriteInteger("global", "nCurPlacedTimes", this.nCurPlacedTimes);
                    }
                    break;
                case "planplacenum"://计划下一摆放位置.
                    {
                        int PlanPlaceNum = Utils.TryToInt32(args.Result[1]);
                        this.txt_plan_placed_num.Text = PlanPlaceNum.ToString();
                    }
                    break;
                case "weightreset"://称重设备置零.
                    {
                        WeightRest();
                    }
                    break;
                case "weightsend"://获取称重重量.
                    {
                        WeightSend();
                    }
                    break;
            }
        }

        // 更新日志显示内容.
        private void updateWarnLogView()
        {
            this.Invoke(new MethodInvoker(delegate ()
            {
                // 显示所有日志数据.
                String strToday = DateTime.Now.ToString("yyyy-MM-dd");
                String strWhere = String.Format("log_date>='{0}'", strToday);
                DataSet list = new ALARM_LOG().GetList(strWhere);
                DataTable dt = list.Tables[0];
                dt.DefaultView.Sort = "id desc";
                dt.Columns.Remove("id");
                dt.Columns.Remove("status");
                dt.Columns["log_date"].ColumnName = "日期";
                dt.Columns["log_time"].ColumnName = "时间";
                dt.Columns["log_classify"].ColumnName = "报警分类";
                dt.Columns["log_content"].ColumnName = "报警内容";

                this.dataGridView1.DataSource = list.Tables[0];
                this.dataGridView1.DefaultCellStyle.ForeColor = Color.Red;
                this.dataGridView1.DefaultCellStyle.Font = new Font("Microsoft YaHei", 8, FontStyle.Bold);
                this.dataGridView1.ColumnHeadersDefaultCellStyle.Font = new Font("Microsoft YaHei", 10, FontStyle.Bold);
            }));
        }

        // 启动机器人
        private void btnStartProduct_Click(object sender, EventArgs e)
        {
            if (MessageBox.Show("您好，确定要启动机器人吗？", "操作确认", MessageBoxButtons.OKCancel) == DialogResult.OK)
            {
                String cmd = "$start,0";
                if (this.epsonSocket1.IsConnection)
                {
                    this.epsonSocket1.tcp_send(cmd);
                }
            }
        }

        // 停止机器人
        private void btnStopProduct_Click(object sender, EventArgs e)
        {
            if (MessageBox.Show("您好，确定要停止机器人吗？", "操作确认", MessageBoxButtons.OKCancel) == DialogResult.OK)
            {
                String cmd = "$rest;";
                if (this.epsonSocket2.IsConnection)
                {
                    // 停止命令发送到5002端口.
                    this.epsonSocket2.tcp_send(cmd);
                }
            }

        }

        // 紧急停止命令发到5000端口.
        private void btnStopNow_Click(object sender, EventArgs e)
        {
            if (this.epsonSocket1.IsConnection)
            {
                String cmd = "$stop";
                this.epsonSocket1.tcp_send(cmd);
            }
        }

        //刷新状态消息.
        private void updateAndroidStatus(string statusMsg)
        {
            if (string.IsNullOrEmpty(statusMsg)) return;
            String[] arr = statusMsg.Split(',');
            if (arr.Length != 2) return;

            String info = "";
            String[] itemNames = { "测试", "示教", "自动", "警告", "严重错误", "安全门", "急停", "错误", "暂停", "运行中", "准备就绪" };
            char[] bits = arr[0].ToCharArray();
            for (int i = 0; i < bits.Length; i++)
            {
                if (bits[i] == '1')
                    info += itemNames[i] + ",";
            }

            // 错误级别需要标红.
            Color color = Color.Black;
            if ((info.IndexOf("警告") >= 0) || (info.IndexOf("严重错误") >= 0) || (info.IndexOf("安全门") >= 0) || (info.IndexOf("急停") >= 0) || (info.IndexOf("错误") >= 0))
            {
                color = Color.Red;
            }

            // 去掉最后的逗号.
            if (info.Length > 0)
            {
                info = info.Remove(info.Length - 1);
                this.lblAndroidStatus.Text = info;
                this.lblAndroidErrorCode.Text = arr[1];
                this.lblAndroidStatus.ForeColor = color;
            }

        }

        // 故障复位.
        private void btnErrorReset_Click(object sender, EventArgs e)
        {
            if (this.epsonSocket1.IsConnection)
            {
                string cmd = "$reset";
                this.epsonSocket1.tcp_send(cmd);
            }
        }


        // 生成绘制矩阵.
        private void ShowMatrix(Model.PRODUCT_ITEM_CONFIG product)
        {
            // 重置所有产品为不可见.
            for (int row = 0; row < maxItem; row++)
            {
                for (int col = 0; col < maxItem; col++)
                {
                    this.mPart[row, col].bVisible = false;
                    this.mlbMatrix[row, col].Reset();
                }
            }

            // 重置所有路径绘制.
            this.mlbDashBorder.Reset();
            this.mlbCenterHole.Reset();

            // 更新当前产品类型.
            this.productType = product.product_type;
            this.boatDashType = product.boat_dish_type;

            // 获取舟皿大小.
            double dash_length = product.boat_dish_length;
            double dash_width = product.boat_dish_width;
            if (this.boatDashType == 2)
            {
                // 圆形舟皿认为长宽是相同的.
                dash_width = dash_length;
            }

            // 获取仿真窗口大小.
            int screen_width = this.picMatirxMain.Width;
            int screen_height = this.picMatirxMain.Height;

            // 确定绘制在仿真窗口的正中心位置.
            double left = (screen_width - dash_length) / 2.0;
            double top = (screen_height - dash_width) / 2.0;
            drawStartpos = new PointF((float)left, (float)top);

            // 根据不同的舟皿类型画出轮廓.
            if (this.boatDashType == 2)
            {
                // 确定舟皿外形.
                this.mlbDashBorder.AddEllipse(new RectangleF((float)left, (float)top, (float)dash_length, (float)dash_length));
            }
            else
            {
                // 确定舟皿外形.
                this.mlbDashBorder.AddRectangle(new RectangleF((float)left, (float)top, (float)dash_length, (float)dash_width));
            }

            double hole_radius = product.central_hole_radius;
            // 确定是否有中心孔,有的话也绘制出来.
            if (hole_radius > 0)
            {
                // 画中心孔.
                double center_x = this.picMatirxMain.Width * 0.5f;
                double center_y = this.picMatirxMain.Height * 0.5f;
                this.mlbDashBorder.AddEllipse(new RectangleF((float)(center_x - hole_radius), (float)(center_y - hole_radius), (float)(hole_radius + hole_radius), (float)(hole_radius + hole_radius)));
            }

            // 产品大小
            float product_length = (float)product.product_length;
            float product_width = (float)product.product_width;

            // 判断当前舟皿可以放多少个产品,默认计算长方形.
            this.nColNo = (short)Math.Floor(dash_length / product_length);
            this.nRowNo = (short)Math.Floor(dash_width / product_width);
            if (productType == 1)
            {
                // 三角形一行可以摆放多少个.
                this.nColNo = (short)Math.Floor((dash_length / (product_length * 0.5f) - 1));
                // 如果是三角形,那么修正产品的高度为以长度为基准.
                product_width = (half_sqrt3 * product_length);
                this.nRowNo = (short)Math.Floor(dash_width / product_width);
            }

            // 如果是槽形舟皿,列数量直接取槽数.
            if (this.boatDashType == 1) this.nRowNo = (short)product.boat_slot_num;
            // 判断一下是不是溢出了.
            float placed_width = (this.nRowNo * product_width);
            if (placed_width > product.boat_dish_width)
            {
                this.picMatirxMain.Refresh();
                MessageBox.Show("对不起,当前摆放大小超出了舟皿大小,请调整参数重试!");
                return;
            }

            // 计算行列摆放次数.
            this.nColPlaceNo = (short)(this.nColNo / product.x_direction_num);
            this.nRowPlaceNo = (short)(this.nRowNo / product.y_direction_num);
            if (product.place_type == (int)ePlacementType.Chakong)
            {
                if (product.matrix_type == (int)eMatirxType.Y2X)
                {
                    if (product.y_direction_num > 1)
                    {
                        this.nRowPlaceNo = (short)((this.nRowNo / (product.y_direction_num * 2)) * 2);
                        int lineSize = ((product.y_direction_num * 2) - 1);
                        if ((this.nRowNo - (this.nRowPlaceNo * product.y_direction_num)) >= lineSize)
                        {
                            // 最后还能多摆一行.
                            this.nRowPlaceNo++;
                        }
                    }
                }
                else if (product.matrix_type == (int)eMatirxType.X2Y)
                {
                    if (product.x_direction_num > 1)
                    {
                        this.nColPlaceNo = (short)((this.nColNo / (product.x_direction_num * 2)) * 2);
                        int lineSize = ((product.x_direction_num * 2) - 1);
                        if ((this.nColNo - (this.nColPlaceNo * product.x_direction_num)) >= lineSize)
                        {
                            // 最后还能多摆一行.
                            this.nColPlaceNo++;
                        }
                    }
                }
            }

            /*
            // 根据摆放次数,重新修正一下行列数量.
            this.nColNo = (short)(this.nColPlaceNo * product.x_direction_num);
            this.nRowNo = (short)(this.nRowPlaceNo * product.y_direction_num);
            */

            // 根据图形类型,确定起始中心点位置,默认是矩形.
            PointF step = GetStepLength(product_length, product_width, productType);
            Vector2d centerPoint = new Vector2d(drawStartpos.X + product_length * 0.5f, drawStartpos.Y + product_width * 0.5f);
            if (1 == productType)
            {
                // 三角形有点特殊,起始点跟矩形有所区别.
                centerPoint = new Vector2d(drawStartpos.X + product_length * 0.5f, drawStartpos.Y + (half_sqrt3 * product_length) * (2.0f / 3));
            }

            //确定是先画行,还是先画列.
            float startX = centerPoint.X;
            for (int row = 0; row < this.nRowNo; row++)
            {
                for (int col = 0; col < this.nColNo; col++)
                {
                    bool bFlip = false;
                    Vector2d tempCenter = new Vector2d(centerPoint.X, centerPoint.Y);
                    // 如果是三角形,需要根据奇偶数进行正反摆放.
                    if ((productType == 1) && ((col % 2) == 1))
                    {
                        // 反向摆,中心点往上移动1/3.
                        tempCenter.Y = tempCenter.Y - (product_width * (1.0f / 3));
                        bFlip = true;
                    }

                    // 添加绘制图形.
                    Vector2d[] points = GetSharpPoints(tempCenter, product_length, product_width, productType, bFlip);
                    AddSharpGraphics(tempCenter, row, col, points, productType, (float)hole_radius, this.boatDashType, (float)dash_length, (float)dash_width);

                    // 判断是否需要绘制中心孔,这是为了测试直接绘制.
                    // this.mlbMatrix[row, col].AddEllipse(new RectangleF(new PointF(tempCenter.X - 2, tempCenter.Y - 2), new SizeF(5, 5)));

                    // 移动到下一个位置.
                    centerPoint.X += step.X;
                }

                // 重置X位置并计算Y的位置.
                centerPoint.X = startX;
                centerPoint.Y += step.Y;
            }

            // modify by James 220417-去除四个边角上的摆件.

            int colSkipNum = Convert.ToInt32(product.boat_first_point_x);

            if ((colSkipNum > 0) && (colSkipNum <= product.x_direction_num) && (product.matrix_type == (int)eMatirxType.X2Y) && (product.x_direction_num > 1))
            {
                if (product.place_type == (int)ePlacementType.Chakong)
                    colSkipNum = 2 * product.x_direction_num;
                else
                    colSkipNum = product.x_direction_num;
            }
            int rowSkipNum = Convert.ToInt32(product.boat_first_point_y);
            if ((rowSkipNum > 0) && (rowSkipNum <= product.y_direction_num) && (product.matrix_type == (int)eMatirxType.Y2X) && (product.y_direction_num > 1))
            {
                if (product.place_type == (int)ePlacementType.Chakong)
                    rowSkipNum = 2 * product.y_direction_num;
                else
                    rowSkipNum = product.y_direction_num;
            }
            // modify by James end.
            this.ClearCornerProduct(this.nColNo, this.nRowNo, colSkipNum, rowSkipNum);
            this.picMatirxMain.Refresh();
        }


        // 清除掉四个角落不摆的产品.
        private void ClearCornerProduct(int nColPlaceNo, int nRowPlaceNo, int colSkipNum, int rowSkipNum)
        {
            do
            {
                // check  if the special input parameter item value.
                if ((colSkipNum <= 0) || (rowSkipNum <= 0)) break;
                if ((nColPlaceNo <= 0) || (nRowPlaceNo <= 0)) break;
                if (((colSkipNum + colSkipNum) >= nColPlaceNo) || ((rowSkipNum + rowSkipNum) >= nRowPlaceNo)) break;

                // add the special four corner item value.
                List<Point> lstCorner = new List<Point>();
                lstCorner.Add(new Point(0, 0)); //LeftTop
                lstCorner.Add(new Point(nColPlaceNo - colSkipNum, 0)); //RightTop
                lstCorner.Add(new Point(nColPlaceNo - colSkipNum, nRowPlaceNo - rowSkipNum)); //RightBottom
                lstCorner.Add(new Point(0, nRowPlaceNo - rowSkipNum)); //LeftBottom

                // loop to set the special corner.
                for (int n = 0; n < lstCorner.Count; n++)
                {
                    Point point = lstCorner[n];
                    for (int row = point.Y; row < point.Y + rowSkipNum; row++)
                    {
                        for (int col = point.X; col < point.X + colSkipNum; col++)
                        {
                            // 隐藏指定的元素.
                            this.mPart[row, col].bVisible = false;
                        }
                    }
                }

            } while (false);
            //Cleanup:
            return;
        }

        // 根据不同的图形,获取对应的绘制点.
        private Vector2d[] GetSharpPoints(Vector2d centerPoint, float product_length, float product_width, int productType, bool bFlip)
        {
            Vector2d[] points = null;
            do
            {
                switch (productType)
                {
                    case 1://三角形
                        {
                            // 这里为了好显示,故意缩小3像素.
                            product_length = product_length - 6;
                            Vector2d rotVector = new Vector2d(0, -(half_sqrt3 * product_length) * (2.0f / 3)); //正向摆放
                            if (bFlip)
                            {
                                rotVector = new Vector2d(0, (half_sqrt3 * product_length) * (2.0f / 3)); //反向摆放.
                            }

                            // 以中心点为中心旋转,得到三角形的三个顶.
                            points = new Vector2d[3];
                            points[0] = centerPoint + rotVector;
                            points[1] = centerPoint + rotVector.Rotate(120);
                            points[2] = centerPoint + rotVector.Rotate(240);
                        }
                        break;
                    default://默认是矩形,只取左上右下两个角.
                        {
                            // 这里为了好显示,估计缩小3像素.
                            float half_length = product_length * 0.5f - 1;
                            float half_width = product_width * 0.5f - 1;
                            points = new Vector2d[4];
                            points[0] = new Vector2d(centerPoint.X - half_length, centerPoint.Y - half_width);
                            points[1] = new Vector2d(centerPoint.X + half_length, centerPoint.Y - half_width);
                            points[2] = new Vector2d(centerPoint.X + half_length, centerPoint.Y + half_width);
                            points[3] = new Vector2d(centerPoint.X - half_length, centerPoint.Y + half_width);
                        }
                        break;
                }

            } while (false);
            //Cleanup:
            return (points);
        }

        // 根据不同的图形,添加绘制图形.
        // hole_radius : 中心孔半径.
        // dashType    : 舟皿类型.
        // dashWidth   : 舟皿宽度.
        // dashHeight  : 舟皿长度.
        private bool AddSharpGraphics(Vector2d centerPoint, int row, int col, Vector2d[] points, int productType, float hole_radius, int dashType, float dashWidth, float dashHeight)
        {
            bool ret = false;
            if (points == null) return false;
            switch (productType)
            {
                case 1://三角形
                    {
                        //判断当前产品是不是落在中心孔里,如果是,那么不要摆放.
                        if ((PointsInDash(points, dashType, dashWidth, dashHeight)) && (!PointsInCenterHole(points, hole_radius)))
                        {
                            // 绘制三角形的三边边.
                            this.mlbMatrix[row, col].AddLine(new PointF(points[0].X, points[0].Y), new PointF(points[1].X, points[1].Y));
                            this.mlbMatrix[row, col].AddLine(new PointF(points[1].X, points[1].Y), new PointF(points[2].X, points[2].Y));
                            this.mlbMatrix[row, col].AddLine(new PointF(points[2].X, points[2].Y), new PointF(points[0].X, points[0].Y));
                            this.mPart[row, col].Center = new PointF((centerPoint.X - drawStartpos.X), (centerPoint.Y - drawStartpos.Y));
                            this.mPart[row, col].bVisible = true;
                            this.nPartsPerTray++;
                            ret = true;
                        }

                    }
                    break;
                default:
                    {
                        float left = points[0].X;
                        float top = points[0].Y;
                        float width = (points[2].X - points[0].X);
                        float height = (points[2].Y - points[0].Y);
                        if ((PointsInDash(points, dashType, dashWidth, dashHeight)) && (!PointsInCenterHole(points, hole_radius)))
                        {
                            this.mlbMatrix[row, col].AddRectangle(new RectangleF(left, top, width, height));
                            this.mPart[row, col].Center = new PointF((centerPoint.X - drawStartpos.X), (centerPoint.Y - drawStartpos.Y));
                            this.mPart[row, col].bVisible = true;
                            this.nPartsPerTray++;
                            ret = true;
                        }
                    }
                    break;
            }
            //Cleanup:
            return ret;
        }

        // 判断所有点会不会落在圆内.
        private bool PointsInCenterHole(Vector2d[] points, float hole_radius)
        {
            bool ret = false;
            if (hole_radius <= 0) return false;
            // 中心点位置.
            float centerX = this.picMatirxMain.Width * 0.5f;
            float centerY = this.picMatirxMain.Height * 0.5f;

            // 循环判断线条是否相交.
            for (int i = 0; i < points.Length; i++)
            {
                float dx = Math.Abs(points[i].X - centerX);
                float dy = Math.Abs(points[i].Y - centerY);
                if ((dx * dx + dy * dy) <= (hole_radius * hole_radius))
                {
                    // 顶点与圆相交,在圆内.
                    ret = true;
                    break;
                }

                // 判断线条是否与圆相交.
                int idxNext = ((i + 1) % points.Length);
                ret = Vector2d.IsCircleIntersectLineSeg(points[i], points[idxNext], new Vector2d(centerX, centerY), hole_radius);
                if (ret) break;
            }
            //Cleanup:
            return ret;
        }

        // 获取图形的外接四边形.
        private RectangleF GetOutLineBox(Vector2d[] points)
        {
            RectangleF rect = new RectangleF(0, 0, 0, 0);
            do
            {
                float minX = 0, minY = 0, maxX = 0, maxY = 0;
                if ((points == null) || (points.Length <= 0)) break;
                minX = maxX = points[0].X;
                minY = maxY = points[0].Y;
                for (int i = 1; i < points.Length; i++)
                {
                    // 获取最小值.
                    if (points[i].X < minX) minX = points[i].X;
                    if (points[i].Y < minY) minY = points[i].Y;
                    // 重置当前位置计数器.
                    this.nCurPlacedTimes = 0;
                    // 重置所有产品颜色为0
                    for (int row = 0; row < maxItem; row++)
                    {
                        for (int col = 0; col < maxItem; col++)
                        {
                            this.mPart[row, col].nColor = 0;
                        }
                    }
                    // 获取最大值.
                    if (points[i].X > maxX) maxX = points[i].X;
                    if (points[i].Y > maxY) maxY = points[i].Y;
                }

                // 获取外接四边形的边框以及大小.
                float width = (maxX - minX);
                float height = (maxY - minY);
                rect.X = minX;
                rect.Y = minY;
                rect.Width = width;
                rect.Height = height;

            } while (false);
            //Cleanup:
            return rect;
        }

        // 判断产品是不是都在舟皿内,需要给出舟皿类型和舟皿大小.
        private bool PointsInDash(Vector2d[] points, int dashType, float dashWidth, float dashHeight)
        {
            bool bin_dash = true; //是否在舟皿内.
            if ((dashWidth <= 0) || (dashHeight <= 0)) return false;
            // 中心点位置.
            float centerX = this.picMatirxMain.Width * 0.5f;
            float centerY = this.picMatirxMain.Height * 0.5f;
            float dash_radius = dashWidth * 0.5f;

            // 根据舟皿类型判断产品是否在舟皿内.
            if (dashType == (int)eBoatDashType.DashCircle) //圆形舟皿.
            {
                // 循环判断线条是否相交.
                for (int i = 0; i < points.Length; i++)
                {
                    float dx = Math.Abs(points[i].X - centerX);
                    float dy = Math.Abs(points[i].Y - centerY);
                    if ((dx * dx + dy * dy) >= (dash_radius * dash_radius))
                    {
                        // 顶点与圆相交,在圆内.
                        bin_dash = false;
                        break;
                    }
                }
            }
            else //长方形.
            {
                Rectangle rect = new Rectangle((int)(centerX - (dashWidth * 0.5f)), (int)(centerY - (dashHeight * 0.5f)), (int)dashWidth, (int)dashHeight);
                // 循环判断线条是否相交.
                for (int i = 0; i < points.Length; i++)
                {
                    if (!rect.Contains(new Point((int)points[i].X, (int)points[i].Y)))
                    {
                        // 顶点与圆相交,在圆内.
                        bin_dash = false;
                        break;
                    }
                }
            }
            //Cleanup:
            return (bin_dash);
        }

        // 获取X,Y方向步长,三角形和矩形这里会有所区别.
        private PointF GetStepLength(float product_length, float product_width, int productType)
        {
            PointF point = new PointF(product_length, product_width);
            if (productType == 1)
            {
                // 三角形.
                point.X = (product_length * 0.5f);
                point.Y = (product_length * half_sqrt3);
            }
            //Cleanup:
            return point;
        }

        // 模拟生产按钮.
        private void btnEmulation_Click(object sender, EventArgs e)
        {
            if (this.bisRunning)
            {
                // 停止仿真.
                this.bisRunning = false;
                this.Timer_EnumationProduct.Interval = 100;
                this.Timer_EnumationProduct.Enabled = false;
                this.Timer_EnumationProduct.Stop();
                this.btnEmulation.Text = "模拟仿真(&P)";

                // 重置当前位置计数器.
                this.nCurPlacedTimes = 0;
                // 重置所有产品颜色为0
                for (int row = 0; row < maxItem; row++)
                {
                    for (int col = 0; col < maxItem; col++)
                    {
                        this.mPart[row, col].nColor = 0;
                    }
                }

                // 刷新空间显示.
                this.picMatirxMain.Refresh();
            }
            else
            {
                // 开始仿真.
                if (this.product == null)
                {
                    MessageBox.Show("对不起,请先选择需要生产的产品!");
                    return;
                }

                // for test only start.
                List<float> lstData = new List<float>();
                String str = GetMatrixPosition("placedatax", ref lstData);
                string cmdx = String.Format("${0};{1}", "pdx", str);
                this.saveLog(cmdx);

                str = GetMatrixPosition("placedatay", ref lstData);
                string cmdy = String.Format("${0};{1}", "pdy", str);
                this.saveLog(cmdy);

                str = GetMatrixPosition("placedatar", ref lstData);
                string cmdz = String.Format("${0};{1}", "pdr", str);
                this.saveLog(cmdz);
                // for test only end.

                // 重新计算当前总共可以摆放多少个,因为要考虑挖孔的情况.
                this.nMaxPlaceTimes = lstPlaceXY.Count;

                // 重置当前位置计数器.
                this.nCurPlacedTimes = 0;
                // 重置所有产品颜色为0
                for (int row = 0; row < maxItem; row++)
                {
                    for (int col = 0; col < maxItem; col++)
                    {
                        this.mPart[row, col].nColor = 0;
                    }
                }


                // 刷新空间显示.
                this.picMatirxMain.Refresh();

                // 更新状态.
                this.bisRunning = true;
                this.btnEmulation.Text = "停止仿真(&P)";
                // 如果是一次摆放多个,为了演示效果,需要设置定时器间隔大一点.
                if ((this.nXDirectionNum > 1) || (this.nYDirectionNum > 1))
                {
                    // 因为摆放得比较快,演示速度可以放慢一点.
                    this.Timer_EnumationProduct.Interval = 300;
                }
                else
                {
                    // 重置演示速度为正常速度.
                    this.Timer_EnumationProduct.Interval = 100;
                }

                // 启动仿真定时器.
                this.Timer_EnumationProduct.Enabled = true;
                this.Timer_EnumationProduct.Start();
            }
        }

        // 更新仿真阵列显示效果.
        private void updateEnumationGraphicsMatrix()
        {
            if (this.nCurPlacedTimes < this.lstPlaceXY.Count)
            {
                Point pos = this.lstPlaceXY[this.nCurPlacedTimes];
                this.mPart[pos.X, pos.Y].nColor = 1; //当前生产颜色.

                // 一次摆放多个,需要处理
                if ((this.nMatrixType == (int)eMatirxType.Y2X) && (this.product.y_direction_num > 1))
                {
                    int nStep = 1;
                    //摆放方式,看是插空还是对齐.
                    if (this.nPlaceType == (int)ePlacementType.Chakong)
                    {
                        nStep = 2;
                    }
                    for (int i = 1; i < this.product.y_direction_num; i++)
                    {
                        int offset = i * nStep;
                        // 查看当前对象的生产颜色.
                        if (this.mPart[pos.X + offset, pos.Y].bVisible)
                            this.mPart[pos.X + offset, pos.Y].nColor = 1;
                    }
                }
                else if ((this.nMatrixType == (int)eMatirxType.X2Y) && (this.product.x_direction_num > 1))
                {
                    int nStep = 1;
                    //摆放方式,看是插空还是对齐.
                    if (this.nPlaceType == (int)ePlacementType.Chakong)
                    {
                        nStep = 2;
                    }
                    for (int i = 1; i < this.product.x_direction_num; i++)
                    {
                        int offset = i * nStep;
                        // 查看当前对象的生产颜色.
                        if (this.mPart[pos.X, pos.Y + offset].bVisible)
                            this.mPart[pos.X, pos.Y + offset].nColor = 1;
                    }
                }
            }

            /*
            int nCurPlacedRowNo = 0; //当前摆放次数行列数.
            int nCurPlacedColNo = 0;  
            int nCurProdStartColNo = 0; //当前摆放位置起始行列数.
            int nCurProdStartRowNo = 0;

            // 根据阵列方式判断当前生产到哪个位置.
            if (this.nMatrixType == (int)eMatirxType.Y2X)
            {
                // 先Y后X.
                // 计算当前摆放次数的行列位置,注意只是摆放次数,不是具体产品的行列值.
                nCurPlacedRowNo = (this.nCurPlacedTimes % this.nRowPlaceNo);
                nCurPlacedColNo = (this.nCurPlacedTimes / this.nRowPlaceNo);

                // 求当前产品的行列下标值.
                nCurProdStartColNo = (nCurPlacedColNo * this.nXDirectionNum); //列不可插空摆放,所以直接乘以x方向数量即可.
                nCurProdStartRowNo = (nCurPlacedRowNo * this.nYDirectionNum);
                if (this.nPlaceType == (int)ePlacementType.Chakong)
                {
                    // 公式法计算先Y后X插空摆放下标位置.
                    nCurProdStartRowNo = ((int)Math.Floor(((double)nCurPlacedRowNo / 2)) * (2 * this.nYDirectionNum - 1) + ((int)Math.Floor(((double)nCurPlacedRowNo + 1) / 2)));
                }

                // 改变当前生产位置模拟块的颜色值.
                for (int i = 0; i < this.nYDirectionNum; i++)
                {
                    // 如果是插空摆放,那么是1,3,5这样,不是按顺序顺下.
                    int offset = i; if (this.nPlaceType == 1) offset = (i * 2);
                    this.mPart[nCurProdStartRowNo + offset, nCurProdStartColNo].nColor = 1; //当前生产颜色.
                }
            }
            else
            {
                // 计算当前摆放次数的行列位置,注意只是摆放次数,不是具体产品的行列值.
                nCurPlacedRowNo = (this.nCurPlacedTimes / this.nColPlaceNo);
                nCurPlacedColNo = (this.nCurPlacedTimes % this.nColPlaceNo);

                // 求当前产品的行列下标值.
                nCurProdStartColNo = (nCurPlacedColNo * this.nXDirectionNum); //列不可插空摆放,所以直接乘以x方向数量即可.
                nCurProdStartRowNo = (nCurPlacedRowNo * this.nYDirectionNum);
                if (this.nPlaceType == (int)ePlacementType.Chakong)
                {
                    // 公式法计算先X后Y插空摆放下标位置.
                    nCurProdStartColNo = ((int)Math.Floor(((double)nCurPlacedColNo / 2)) * (2 * this.nXDirectionNum - 1) + ((int)Math.Floor(((double)nCurPlacedColNo + 1) / 2)));
                }

                // 改变当前生产位置模拟块的颜色值.
                for (int i = 0; i < this.nXDirectionNum; i++)
                {
                    // 如果是插空摆放,那么是1,3,5这样,不是按顺序顺下.
                    int offset = i; if (this.nPlaceType == 1) offset = (i * 2);
                    this.mPart[nCurProdStartRowNo, nCurProdStartColNo + offset].nColor = 1; //当前生产颜色.
                }
            }
            */

            // 刷新显示.
            this.picMatirxMain.Refresh();
        }

        // 模拟生产.
        private void Timer_EnumationProduct_Tick(object sender, EventArgs e)
        {
            // 更新显示阵列.
            this.updateEnumationGraphicsMatrix();

            // 移动到下一位置.
            this.nCurPlacedTimes++;

            // 判断是不是全部模拟完毕.
            if (this.nCurPlacedTimes >= this.nMaxPlaceTimes)
            {
                btnEmulation_Click(sender, e);
                MessageBox.Show("恭喜,模拟摆放结束!");
            }
        }

        // Timer_GetStatus定时器响应函数.
        private void Timer_Getstatus_Tick(object sender, EventArgs e)
        {
            if (this.epsonSocket1.IsConnection)
            {
                this.epsonSocket1.sendGetStatus();
            }
        }

        // whichAxis : 如果只是生成坐标序列,这个参数对应,0:x,1:y,2:z.
        private String GetMatrixPosition(String whichAxis, ref List<float> lstData)
        {
            String str = "";
            int nCurProdStartColNo = 0; //列不可插空摆放,所以直接乘以x方向数量即可.
            int nCurProdStartRowNo = 0;

            lstData.Clear();
            if ("placedatax" == whichAxis) lstPlaceXY.Clear(); // 只在x坐标时保存摆放位置.
            int nMaxPlaceTimes = this.nColPlaceNo * this.nRowPlaceNo;
            for (int nPlacedTimes = 0; nPlacedTimes < nMaxPlaceTimes; nPlacedTimes++)
            {
                int nCurPlacedRowNo = 0;
                int nCurPlacedColNo = 0;
                // 根据阵列方式判断当前生产到哪个位置.
                if (this.nMatrixType == (int)eMatirxType.Y2X)
                {
                    // 先Y后X.
                    // 计算当前摆放次数的行列位置,注意只是摆放次数,不是具体产品的行列值.
                    nCurPlacedRowNo = (nPlacedTimes % this.nRowPlaceNo);
                    nCurPlacedColNo = (nPlacedTimes / this.nRowPlaceNo);

                    // 求当前产品的行列下标值.
                    nCurProdStartColNo = (nCurPlacedColNo * this.nXDirectionNum); //列不可插空摆放,所以直接乘以x方向数量即可.
                    nCurProdStartRowNo = (nCurPlacedRowNo * this.nYDirectionNum);
                    if (this.nPlaceType == (int)ePlacementType.Chakong)
                    {
                        // 公式法计算插空摆放下标位置.
                        nCurProdStartRowNo = ((int)Math.Floor(((double)nCurPlacedRowNo / 2)) * (2 * this.nYDirectionNum - 1) + ((int)Math.Floor(((double)nCurPlacedRowNo + 1) / 2)));
                    }
                }
                else
                {
                    // 计算当前摆放次数的行列位置,注意只是摆放次数,不是具体产品的行列值.
                    nCurPlacedRowNo = (nPlacedTimes / this.nColPlaceNo);
                    nCurPlacedColNo = (nPlacedTimes % this.nColPlaceNo);

                    // 求当前产品的行列下标值.
                    nCurProdStartColNo = (nCurPlacedColNo * this.nXDirectionNum); //列不可插空摆放,所以直接乘以x方向数量即可.
                    nCurProdStartRowNo = (nCurPlacedRowNo * this.nYDirectionNum);
                    if (this.nPlaceType == (int)ePlacementType.Chakong)
                    {
                        // 公式法计算先X后Y插空摆放下标位置.
                        nCurProdStartColNo = ((int)Math.Floor(((double)nCurPlacedColNo / 2)) * (2 * this.nXDirectionNum - 1) + ((int)Math.Floor(((double)nCurPlacedColNo + 1) / 2)));
                    }
                }

                // 获取当前产品下标对应的mPart对象,并获取其相对坐标值.
                defPart mPart = this.mPart[nCurProdStartRowNo, nCurProdStartColNo];
                if (mPart.bVisible)
                {
                    // 记录当前坐标下的摆放位置.
                    if ("placedatax" == whichAxis)
                    {
                        lstPlaceXY.Add(new Point(nCurProdStartRowNo, nCurProdStartColNo));
                    }

                    // 计算具体摆放坐标值.
                    PointF start = mPart.Center;
                    if (this.boatDashType == (int)eBoatDashType.DashCircle)
                    {
                        // 圆形舟,调教的是中心点,那么减去中心点的坐标即是坐标原点.
                        // start.X  = this.startAndroidX - start.X;
                        // start.Y  = this.startAndroidY + start.Y;
                    }
                    else
                    {
                        // 三角形减去一个产品中心点即是坐标原点0,0
                        if (this.productType == (int)eProductType.Triangle)
                        {
                            start.X -= (float)(this.product.product_length * 0.5f);
                            start.Y -= (float)((half_sqrt3 * this.product.product_length) * (2.0f / 3));
                        }
                        else //其他按四边形算,减去半个产品大小.
                        {
                            start.X -= (float)(this.product.product_length * 0.5f);
                            start.Y -= (float)(this.product.product_width * 0.5f);
                        }
                    }

                    /*
                    float offsetX = (float)(nCurProdStartColNo * this.product.product_length);
                    float offsetY = (float)(nCurProdStartRowNo * this.product.product_width);
                    if (this.boatDashType == (int)eBoatDashType.DashCircle)
                    {
                        // 如果是圆形舟皿,需要加上1/2产品大小才是产品中心点.
                        offsetX += (float)this.product.product_length * 0.5f;
                        offsetY += (float)this.product.product_width  * 0.5f;
                    }

                    // PointF pos = new PointF(this.startAndroidX + mPart.Center.X, this.startAndroidY + mPart.Center.Y);
                    PointF pos = new PointF(this.startAndroidX + offsetX, this.startAndroidY + offsetY);
                    */

                    PointF pos = new PointF(this.startAndroidX - start.X, this.startAndroidY + start.Y);
                    switch (whichAxis)
                    {
                        case "placedatax"://X
                            str += String.Format("{0:F1};", pos.X);
                            lstData.Add(pos.X);
                            break;
                        case "placedatay"://Y
                            str += String.Format("{0:F1};", pos.Y);
                            lstData.Add(pos.Y);
                            break;
                        default://默认不支持旋转,给0.
                            {
                                int val = 0;
                                if ((this.productType == (int)eProductType.Triangle) && ((nCurProdStartColNo % 2) == 1))
                                {
                                    val = 1;
                                }
                                // add the special value now.
                                str += val.ToString() + ";";
                                // add to list.
                                lstData.Add(val);
                            }
                            break;
                    }
                }
            }
            //Cleanup:
            return (str);
        }

        // 显示机器人IO状态.
        private void btnAndroidIO_Click(object sender, EventArgs e)
        {
            Android_IO_Form form = new Android_IO_Form(this.epsonSocket1);
            form.ShowDialog(this);
        }

        List<PCHMI.TIRGGER> lstTrigger = new List<PCHMI.TIRGGER>();
        defPLCTrigger[] def_plc_triggers = new defPLCTrigger[]
        {

            new defPLCTrigger("M301","上升沿","PLC报警","原点回归中"),
            new defPLCTrigger("M302","上升沿","PLC报警","上次位置回归中"),
            new defPLCTrigger("M303","上升沿","PLC报警","机器人急停"),
            new defPLCTrigger("M304","上升沿","PLC报警","伺服报警"),
            new defPLCTrigger("M305","上升沿","PLC报警","伺服未使能"),
            new defPLCTrigger("M306","上升沿","PLC报警","轴1限位"),
            new defPLCTrigger("M307","上升沿","PLC报警","勾舟2不在原位"),
            new defPLCTrigger("M308","上升沿","PLC报警","勾舟1不在原位"),
            new defPLCTrigger("M309","上升沿","PLC报警","舟皿输送不在原位"),
            new defPLCTrigger("M310","上升沿","PLC报警","机器人开始运行"),
            new defPLCTrigger("M311","上升沿","PLC报警","升舟皿后库位无舟皿"),
            new defPLCTrigger("M312","上升沿","PLC报警","库位已满"),
            new defPLCTrigger("M313","上升沿","PLC报警","库位有舟皿无法放舟"),
            new defPLCTrigger("M314","上升沿","PLC报警","摆舟位未检测到舟皿"),
        };

        // 初始化生成所有的Trigger.
        private void initTriggerList()
        {
            for (int i = 0; i < def_plc_triggers.Length; i++)
            {
                defPLCTrigger item = def_plc_triggers[i];
                lstTrigger.Add(new PCHMI.TIRGGER(0, item.addr, item.type));
            }
        }
        //手动换舟
        PCHMI.TIRGGER m205 = new PCHMI.TIRGGER(0, "M205", "上升沿");
        //外接启动按钮
        PCHMI.TIRGGER m211 = new PCHMI.TIRGGER(0, "M211", "上升沿");
        //外接报警复位按钮
        PCHMI.TIRGGER m212 = new PCHMI.TIRGGER(0, "M212", "上升沿");
        //外接紧急停止按钮
        PCHMI.TIRGGER m218 = new PCHMI.TIRGGER(0, "M218", "上升沿");
        //外接停止按钮
        PCHMI.TIRGGER m219 = new PCHMI.TIRGGER(0, "M219", "上升沿");
       
        private void Timer_PLCTick_Tick(object sender, EventArgs e)
        {
            for (int i = 0; i < lstTrigger.Count; i++)
            {
                defPLCTrigger item = def_plc_triggers[i];
                if (lstTrigger[i].Enable)
                {
                    ZWGXPICK_SYS.Model.ALARM_LOG model = new ZWGXPICK_SYS.Model.ALARM_LOG();
                    model.log_date = DateTime.Now.ToString("yyyy-MM-dd");
                    model.log_time = DateTime.Now.ToString("HH:mm:ss");
                    model.log_classify = item.logclassify;
                    model.log_content = item.logcontent;
                    new ALARM_LOG().Add(model);
                    updateWarnLogView();
                }
            }

            //手动换舟后下一摆放位置重置
            if (m205.Enable)
            {
                this.txt_plan_placed_num.Text = "1";
                IniFile ini = new IniFile(PubConstant.config_file);
                ini.WriteString("global", "txt_plan_placed_num", "1");
            }
            //外接启动按钮
            if (m211.Enable)
            {
                String cmd = "$start,0";
                if (this.epsonSocket1.IsConnection)
                {
                    this.epsonSocket1.tcp_send(cmd);
                }
            }
            //外接报警复位按钮
            if (m212.Enable)
            {
                if (this.epsonSocket1.IsConnection)
                {
                    string cmd = "$reset";
                    this.epsonSocket1.tcp_send(cmd);
                }
            }
            //外接紧急停止按钮
            if (m218.Enable)
            {
                if (this.epsonSocket1.IsConnection)
                {
                    String cmd = "$stop";
                    this.epsonSocket1.tcp_send(cmd);
                }
            }

            //外接停止按钮
            if (m219.Enable)
            {
                String cmd = "$rest;";
                if (this.epsonSocket2.IsConnection)
                {
                    // 停止命令发送到5002端口.
                    this.epsonSocket2.tcp_send(cmd);
                }
            }
        }

        // 重置下一摆放位置参数.
        private void btn_resetPlaceTimes_Click(object sender, EventArgs e)
        {
            this.txt_plan_placed_num.Text = "1";
            IniFile ini = new IniFile(PubConstant.config_file);
            ini.WriteString("global", "txt_plan_placed_num", "1");
        }

        // 称重频率不能为0
        private void txt_weight_interval_Leave(object sender, EventArgs e)
        {
            int nWeight = Utils.TryToInt32(this.txt_weight_interval.Text);
            if (nWeight <= 0)
            {
                this.txt_weight_interval.Text = "1";
            }
        }

        // 清模频率.
        private void txt_cleanmode_interval_Leave(object sender, EventArgs e)
        {
            int nClearMode = Utils.TryToInt32(this.txt_cleanmode_interval.Text);
            if (nClearMode <= 0)
            {
                this.txt_cleanmode_interval.Text = "1";
            }
        }

        // 连接称重不合格.
        private void txt_continuity_unqualified_Leave(object sender, EventArgs e)
        {
            int nContinuity = Utils.TryToInt32(this.txt_continuity_unqualified.Text);
            if (nContinuity <= 0)
            {
                this.txt_continuity_unqualified.Text = "1";
            }
        }

        // 创建日志目录.
        private void createLogDir()
        {
            string logDir = System.Environment.CurrentDirectory + "\\Logs";
            Directory.CreateDirectory(logDir);
        }

        // 保存日志内容.
        private void saveLog(string str)
        {
            string logDir = System.Environment.CurrentDirectory + "\\Logs\\";
            string logFile = logDir + DateTime.Today.ToString("yyyy-MM-dd") + ".log";
            string text = "【" + DateTime.Now.ToLongTimeString() + "】" + str + Environment.NewLine;
            try
            {
                FileStream fs = File.Open(logFile, FileMode.Append);
                StreamWriter sw = new StreamWriter(fs);
                sw.Write(text);
                sw.Close();
                fs.Close();
            }
            catch (Exception ex) { }
        }

        #region 称重相关
        //称重设备配置界面
        private void btnWeightConfig_Click(object sender, EventArgs e)
        {
            frmWeightPort form = new frmWeightPort();
            form.ShowDialog(this);
        }

        public void InitWeightEquipment()
        {
            IniFile ini = new IniFile(PubConstant.config_file);
            //重量计算参数
            setSingleUpperLimit = decimal.Parse(ini.ReadString("weight_RS232", "setSingleUpperLimit", "0"));
            setAdjustmentUpperLimit = decimal.Parse(ini.ReadString("weight_RS232", "setAdjustmentUpperLimit", "0"));
            setStandardWeight = decimal.Parse(ini.ReadString("weight_RS232", "setStandardWeight"    ));
            setAdjustmentLowerLimit = decimal.Parse(ini.ReadString("weight_RS232", "setAdjustmentLowerLimit", "0"));
            setSingleLowerLimit = decimal.Parse(ini.ReadString("weight_RS232", "setSingleLowerLimit", "0"));
            if (!weightEquipment.IsOpen())
            {
                var baud = ini.ReadInteger("weight_RS232", "baud", 0);
                var comName = ini.ReadString("weight_RS232", "comName");
                weightEquipment.Open(comName, baud);
            }
            
        }

        //称重置零
        private void WeightRest()
        {
            var result = false;
            if (!weightEquipment.IsOpen())
            {
                InitWeightEquipment();
                result = weightEquipment.Open();
            }
            if (this.epsonSocket2.IsConnection)
            {
                string cmd = $"$weightreset;{(result ? 1 : 0)}";
                this.epsonSocket2.tcp_send(cmd);
                
                WeightPrintMessage($"发送【称重置零】命令：{cmd} 【{(result ? "成功" : "失败")}】");

                PLCWriteBool("M215", false);
                PLCWriteBool("M216", false);
            }
        }

        //获取称重
        private void WeightSend()
        {
            var result = CalculatingWeighingResult();
            if (this.epsonSocket2.IsConnection)
            {
                string cmd = $"$weightsend;{(result ? 1 : 0)}";
                this.epsonSocket2.tcp_send(cmd);

                WeightPrintMessage($"发送【称重结果】命令：{cmd}  【{(result ? "合格" : "不合格")}】");
                //关闭连接
                weightEquipment.Close();

                WeightAddToDB(result);
            }
        }

        //计算称重结果
        private bool CalculatingWeighingResult()
        {
            var result = false;
            var txtSetStandardWeight = 0m;
            this.Invoke(new MethodInvoker(() =>
            {
                decimal.TryParse(txt_standard_weight.Text.Trim(), out txtSetStandardWeight);
            }));
            if (txtSetStandardWeight <= 0)
            {
                return false;
            }
            //调整下限<实时重量<标准重量，返回称重OK给机械手
            if (setAdjustmentLowerLimit <= txtSetStandardWeight && txtSetStandardWeight <= setStandardWeight)
            {
                result = true;
            }
            //标准重量 < 实时重量 < 调整上限，返回称重OK给机械手
            else if (setStandardWeight <= txtSetStandardWeight && txtSetStandardWeight <= setAdjustmentUpperLimit)
            {
                result = true;
            }
            //单重下限<实时重量<调整下限，返回称重OK给机械手，同时将PLC M216写TRUE
            else if (setSingleLowerLimit <= txtSetStandardWeight && txtSetStandardWeight <= setAdjustmentLowerLimit)
            {
                result = true;
                PLCWriteBool("M216", true);
            }
            // 调整上限<实时重量<单重上限，返回称重OK给机械手，同时将PLC M215写TRUE
            else if (setAdjustmentUpperLimit <= txtSetStandardWeight && txtSetStandardWeight <= setSingleUpperLimit)
            {
                result = true;
                PLCWriteBool("M215",true);
            }
            //调实时重量<调整下限，返回称重NG给机械手
            else if (txtSetStandardWeight <= setAdjustmentLowerLimit)
            {
                result = false;
            }
            //调实时重量>调整上限，返回称重NG给机械手
            else if (txtSetStandardWeight >= setAdjustmentUpperLimit)
            {
                result = false;
            }
           
            return result;
        }

        //将重量信息添加到数据库
        private void WeightAddToDB(bool result)
        {
            try
            {
                // 称重数据.
                ZWGXPICK_SYS.Model.WEIGHT_LOG weight = new Model.WEIGHT_LOG();
                weight.product_model = product?.product_name;
                weight.product_date = DateTime.Now.ToString("yyyy-MM-dd");
                weight.product_time = DateTime.Now.ToString("HH:mm:ss");
                weight.weight = "#";
                weight.is_qualified = result ? 1 : 0;
                new WEIGHT_LOG().Add(weight);
            }
            catch (Exception ex)
            {
                WeightPrintMessage($"将重量信息保存到数据库失败，{ex.Message}");
            }

        }

        //称重设备返回重量信息
        private void WeightEquipment_WeightSend(object sender, WeightModel e)
        {
            this.Invoke(new MethodInvoker(() => 
            {
                txt_standard_weight.Text = e.Weight.ToString();
            }));
        }

        //称重设备错误信息
        private void WeightEquipment_MessageSend(object sender, MessageEventArgs e)
        {
            WeightPrintMessage(e.Message);
        }

        //称重重置
        private void btnWeightRest_Click(object sender, EventArgs e)
        {
            WeightRest();
        }

        private void btnGetWeight_Click(object sender, EventArgs e)
        {
            WeightSend();
        }

        //称重计算
        private void btnWeightCalculate_Click(object sender, EventArgs e)
        {
            var  result = CalculatingWeighingResult();
            WeightPrintMessage($"手动称重【称重结果】：【{(result ? "合格" : "不合格")}】");
        }

        private void PLCWriteBool(string pointName,bool value)
        {
            var res = PLCCommunicationFactory.Instance.Write(pointName, value);
            var msg = "";
            if (res.IsSuccess)
            {
                msg = $"{pointName} 写入值 {value} 成功!";
            }
            else
            {
                msg = $"{pointName} 写入值  {value} 失败!{res.Message}";
            }
            WeightPrintMessage(msg);
        }

        //输出称重消息
        private void WeightPrintMessage(string message)
        {

            if (txtWeightMessage.Lines.Length >= 500)
            {
                txtWeightMessage.Clear();
            }
            txtWeightMessage.AppendText($"【{DateTime.Now.ToString("HH:mm:ss")}】：{message}{Environment.NewLine}");
        }

        #endregion


    }
}
